@@ -11,7 +11,7 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  $Id: lo.h,v 1.7 2005/02/28 14:33:19 theno23 Exp $
+ *  $Id: lo.h,v 1.8 2005/08/09 15:06:04 mhabets Exp $
  */
 
 #ifndef LO_H
@@ -158,6 +158,16 @@ void lo_server_thread_free(lo_server_thread st);
 lo_method lo_server_thread_add_method(lo_server_thread st, const char *path,
                                const char *typespec, lo_method_handler h,
                                void *user_data);
+/**
+ * \brief Delete an OSC method from the specifed server thread.
+ *
+ * \param st The server thread the method is to be removed from.
+ * \param path The OSC path of the method to delete. If NULL is passed the
+ * method will match the generic handler.
+ * \param typespec The typespec the method accepts.
+ */
+void lo_server_thread_del_method(lo_server_thread st, const char *path,
+				 const char *typespec);
 
 /**
  * \brief Start the server thread
