@@ -11,7 +11,7 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  $Id: lo_endian.h.in,v 1.3 2004/11/06 14:49:45 trutkin Exp $
+ *  $Id: lo_endian.h.in,v 1.4 2005/04/25 21:46:47 xovo Exp $
  */
 
 #ifndef LO_ENDIAN_H
@@ -19,7 +19,13 @@
 
 #include <sys/types.h>
 #include <stdint.h>
+
+#ifdef WIN32
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#else
 #include <netinet/in.h>
+#endif
 
 #ifdef __cplusplus
 extern "C" {
