@@ -11,7 +11,7 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  $Id: lo_lowlevel.h,v 1.10 2005/01/19 07:59:49 theno23 Exp $
+ *  $Id: lo_lowlevel.h,v 1.11 2005/08/09 15:06:04 mhabets Exp $
  */
 
 #ifndef LO_LOWLEVEL_H
@@ -395,6 +395,17 @@ lo_method lo_server_add_method(lo_server s, const char *path,
                                const char *typespec, lo_method_handler h,
                                void *user_data);
 
+/**
+ * \brief Delete an OSC method from the specifed server.
+ *
+ * \param s The server the method is to be removed from.
+ * \param path The OSC path of the method to delete. If NULL is passed the
+ * method will match the generic handler.
+ * \param typespec The typespec the method accepts.
+ */
+void lo_server_del_method(lo_server s, const char *path,
+                               const char *typespec);
+
 /**
  * \brief Return the file descriptor of the server socket.
  *
