@@ -11,12 +11,14 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  $Id: lo_osc_types.h,v 1.2 2004/11/04 10:33:36 theno23 Exp $
+ *  $Id: lo_osc_types.h,v 1.5 2005/09/02 22:41:07 theno23 Exp $
  */
 
 #ifndef LO_OSC_TYPES_H
 #define LO_OSC_TYPES_H
 
+#include <stdint.h>
+
 /**
  * \addtogroup liblo
  * @{
@@ -105,7 +107,7 @@ typedef union {
 	  * distinguish strings and symbols. */
     char       S;
 	/** Standard C, 8 bit, char. */
-    char       c;
+    unsigned char c;
 	/** A 4 byte MIDI packet. */
     uint8_t    m[4];
 	/** OSC TimeTag value. */
