@@ -11,19 +11,29 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  $Id: send.c,v 1.9 2005/03/02 16:54:43 theno23 Exp $
+ *  $Id: send.c,v 1.14 2005/09/02 22:41:07 theno23 Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdarg.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <netdb.h>
 #include <errno.h>
 #include <unistd.h>
 #include <sys/types.h>
+
+#ifdef WIN32
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#else
+#include <netdb.h>
 #include <sys/socket.h>
 #include <sys/un.h>
+#endif
 
 #include "lo_types_internal.h"
 #include "lo/lo.h"
@@ -32,6 +42,16 @@
 #define MSG_NOSIGNAL 0
 #endif
 
+#ifdef WIN32
+int initWSock();
+#endif
+
+#ifdef WIN32
+#define geterror() WSAGetLastError()
+#else
+#define geterror() errno
+#endif
+
 static int resolve_address(lo_address a);
 
 static void add_varargs(lo_address t, lo_message m, va_list ap,
@@ -294,15 +314,14 @@ static int resolve_address(lo_address a)
 	struct addrinfo *ai;
 	struct addrinfo hints;
 
-	hints.ai_flags = 0;
-	hints.ai_family = PF_UNSPEC;
+	memset(&hints, 0, sizeof(hints));
+#ifdef DISABLE_IPV6
+	hints.ai_family = PF_INET;
+#else
+    hints.ai_family = PF_UNSPEC;
+#endif	
 	hints.ai_socktype = a->proto == LO_UDP ? SOCK_DGRAM : SOCK_STREAM;
-	hints.ai_protocol = 0;
-	hints.ai_addrlen = 0;
-	hints.ai_canonname = NULL;
-	hints.ai_addr = NULL;
-	hints.ai_next = NULL;
-
+	
 	if ((ret = getaddrinfo(a->host, a->port, &hints, &ai))) {
 	    a->errnum = ret;
 	    a->errstr = gai_strerror(ret);
@@ -325,7 +344,7 @@ static int resolve_address(lo_address a)
 	a->socket = socket(ai->ai_family, hints.ai_socktype, 0);
 
 	if ((ret = connect(a->socket, a->ai->ai_addr, a->ai->ai_addrlen))) {
-	    a->errnum = errno;
+	    a->errnum = geterror();
 	    a->errstr = NULL;
 	    //XXX freeaddrinfo(a->ai);
 
@@ -334,12 +353,14 @@ static int resolve_address(lo_address a)
 #if 0
 	}
 #endif
-    } else if (a->proto == LO_UNIX) {
+    }
+#ifndef WIN32
+    else if (a->proto == LO_UNIX) {
 	struct sockaddr_un sa;
 
 	a->socket = socket(PF_UNIX, SOCK_DGRAM, 0);
 	if (a->socket == -1) {
-	    a->errnum = errno;
+	    a->errnum = geterror();
 	    a->errstr = NULL;
 
 	    return -1;
@@ -350,13 +371,15 @@ static int resolve_address(lo_address a)
 
 	if ((ret = connect(a->socket, (struct sockaddr *)&sa,
 			sizeof(sa))) < 0) {
-	    a->errnum = errno;
+	    a->errnum = geterror();
 	    a->errstr = NULL;
 
 	    return -1;
 	}
 	a->ai = (void *)1;
-    } else {
+    } 
+#endif
+    else {
 	/* unkonwn proto */
 
 	return -2;
@@ -371,6 +394,10 @@ int lo_send_message(lo_address a, const char *path, lo_message msg)
     char *data;
     int ret;
 
+#ifdef WIN32
+    if(!initWSock()) return -1;
+#endif
+
     if (lo_strsize(path) + lo_strsize(msg->types) + msg->datalen >
 	LO_MAX_MSG_SIZE) {
 	a->errnum = 99;
@@ -408,7 +435,7 @@ int lo_send_message(lo_address a, const char *path, lo_message msg)
     free(data);
 
     if (ret == -1) {
-	a->errnum = errno;
+	a->errnum = geterror();
 	a->errstr = NULL;
     } else {
 	a->errnum = 0;
@@ -424,6 +451,10 @@ int lo_send_bundle(lo_address a, lo_bundle b)
     char *data;
     int ret;
 
+#ifdef WIN32
+    if(!initWSock()) return -1;
+#endif
+
     if (data_len > LO_MAX_MSG_SIZE) {
 	a->errnum = 99;
 	a->errstr = "Attempted to send bundle in excess of maximum "
@@ -460,7 +491,7 @@ int lo_send_bundle(lo_address a, lo_bundle b)
     free(data);
 
     if (ret == -1) {
-	a->errnum = errno;
+	a->errnum = geterror();
 	a->errstr = NULL;
     } else {
 	a->errnum = 0;
