@@ -11,21 +11,38 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  $Id: server.c,v 1.17 2005/03/02 14:53:17 theno23 Exp $
+ *  $Id: server.c,v 1.25 2005/09/02 22:41:07 theno23 Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
-#include <netdb.h>
 #include <string.h>
 #include <errno.h>
 #include <float.h>
 #include <sys/types.h>
+
+#ifdef WIN32
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#define EADDRINUSE WSAEADDRINUSE
+#else
+#include <netdb.h>
 #include <sys/socket.h>
 #include <sys/poll.h>
 #include <sys/un.h>
 #include <arpa/inet.h>
+#endif
+
+#ifdef WIN32
+#define geterror() WSAGetLastError()
+#else
+#define geterror() errno
+#endif
 
 #include "lo_types_internal.h"
 #include "lo/lo.h"
@@ -49,11 +66,70 @@ static void dispatch_method(lo_server s, const char *path, char *types,
 static int dispatch_queued(lo_server s);
 static void queue_data(lo_server s, lo_timetag ts, void *data, size_t len);
 
+
+#ifdef WIN32
+// Copied from the Win32 SDK 
+
+// WARNING: The gai_strerror inline functions below use static buffers,
+// and hence are not thread-safe.  We'll use buffers long enough to hold
+// 1k characters.  Any system error messages longer than this will be
+// returned as empty strings.  However 1k should work for the error codes
+// used by getaddrinfo().
+#define GAI_STRERROR_BUFFER_SIZE 1024
+
+char *WSAAPI gai_strerrorA(int ecode)
+{
+    DWORD dwMsgLen;
+    static char buff[GAI_STRERROR_BUFFER_SIZE + 1];
+
+    dwMsgLen = FormatMessageA(FORMAT_MESSAGE_FROM_SYSTEM
+                             |FORMAT_MESSAGE_IGNORE_INSERTS
+                             |FORMAT_MESSAGE_MAX_WIDTH_MASK,
+                              NULL,
+                              ecode,
+                              MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
+                              (LPSTR)buff,
+                              GAI_STRERROR_BUFFER_SIZE,
+                              NULL);
+    return buff;
+}
+
+
+static int stateWSock = -1;
+
+int initWSock()
+{
+    if(stateWSock >= 0) return stateWSock;
+    /* TODO - which version of Winsock do we actually need? */
+
+    WORD reqversion = MAKEWORD( 2, 2 );
+    WSADATA wsaData;
+    if(WSAStartup(reqversion,&wsaData) != 0) {
+        /* Couldn't initialize Winsock */
+        stateWSock = 0;
+    }
+    else if ( LOBYTE( wsaData.wVersion ) != LOBYTE(reqversion) ||
+            HIBYTE( wsaData.wVersion ) != HIBYTE(reqversion) ) {
+        /* wrong version */
+        WSACleanup();
+        stateWSock = 0; 
+    }
+    else
+        stateWSock = 1;
+
+    return stateWSock;
+}
+#endif
+
 lo_server lo_server_new(const char *port, lo_err_handler err_h)
 {
+#ifndef WIN32
     if (port && *port == '/') {
 	return lo_server_new_with_proto(port, LO_UNIX, err_h);
-    } else {
+    } 
+    else 
+#endif
+    {
 	return lo_server_new_with_proto(port, LO_UDP, err_h);
     }
 }
@@ -61,7 +137,7 @@ lo_server lo_server_new(const char *port, lo_err_handler err_h)
 lo_server lo_server_new_with_proto(const char *port, int proto,
 				   lo_err_handler err_h)
 {
-    lo_server s = calloc(1, sizeof(struct _lo_server));
+    lo_server s;
     struct addrinfo *ai = NULL, *it, *used;
     struct addrinfo hints;
     int ret = -1;
@@ -70,6 +146,12 @@ lo_server lo_server_new_with_proto(const char *port, int proto,
     const char *service;
     char hostname[LO_HOST_SIZE];
 
+#ifdef WIN32
+    if(!initWSock()) return NULL;
+#endif
+    
+    s = calloc(1, sizeof(struct _lo_server));
+
     s->err_h = err_h;
     s->first = NULL;
     s->ai = NULL;
@@ -78,18 +160,25 @@ lo_server lo_server_new_with_proto(const char *port, int proto,
     s->port = 0;
     s->path = NULL;
     s->queued = NULL;
+    s->socket = -1;
+
+    memset(&hints, 0, sizeof(hints));
 
     if (proto == LO_UDP) {
 	hints.ai_socktype = SOCK_DGRAM;
     } else if (proto == LO_TCP) {
 	hints.ai_socktype = SOCK_STREAM;
-    } else if (proto == LO_UNIX) {
+    } 
+#ifndef WIN32
+    else if (proto == LO_UNIX) {
+
 	struct sockaddr_un sa;
 
 	s->socket = socket(PF_UNIX, SOCK_DGRAM, 0);
 	if (s->socket == -1) {
+        int err = geterror();
 	    used = NULL;
-	    lo_throw(s, errno, strerror(errno), "socket()");
+	    lo_throw(s, err, strerror(err), "socket()");
 	    lo_server_free(s);
 
 	    return NULL;
@@ -99,7 +188,8 @@ lo_server lo_server_new_with_proto(const char *port, int proto,
 	strncpy(sa.sun_path, port, 107);
 
 	if ((ret = bind(s->socket, (struct sockaddr *)&sa, sizeof(sa))) < 0) {
-	    lo_throw(s, errno, strerror(errno), "bind()");
+        int err = geterror();      
+	    lo_throw(s, err, strerror(err), "bind()");
 
 	    lo_server_free(s);
 	    return NULL;
@@ -108,20 +198,21 @@ lo_server lo_server_new_with_proto(const char *port, int proto,
 	s->path = strdup(port);
 
 	return s;
-    } else {
+    } 
+#endif
+    else {
 	lo_throw(s, LO_UNKNOWNPROTO, "Unknown protocol", NULL);
 	lo_server_free(s);
 
 	return NULL;
     }
 
-    hints.ai_flags = AI_PASSIVE;
+#ifdef DISABLE_IPV6
+    hints.ai_family = PF_INET;
+#else
     hints.ai_family = PF_UNSPEC;
-    hints.ai_protocol = 0;
-    hints.ai_addrlen = 0;
-    hints.ai_canonname = NULL;
-    hints.ai_addr = NULL;
-    hints.ai_next = NULL;
+#endif
+    hints.ai_flags = AI_PASSIVE;
 
     if (!port) {
 	service = pnum;
@@ -156,20 +247,22 @@ lo_server lo_server_new_with_proto(const char *port, int proto,
 	    s->socket = socket(it->ai_family, hints.ai_socktype, 0);
 	}
 	if (s->socket == -1) {
+        int err = geterror();
 	    used = NULL;
-	    lo_throw(s, errno, strerror(errno), "socket()");
+	    lo_throw(s, err, strerror(err), "socket()");
 
 	    lo_server_free(s);
 	    return NULL;
 	}
 
 	if ((ret = bind(s->socket, used->ai_addr, used->ai_addrlen)) < 0) {
-	    if (errno == EINVAL || errno == EADDRINUSE) {
+        int err = geterror();
+	    if (err == EINVAL || err == EADDRINUSE) {
 		used = NULL;
 
 		continue;
 	    }
-	    lo_throw(s, errno, strerror(errno), "bind()");
+	    lo_throw(s, err, strerror(err), "bind()");
 
 	    lo_server_free(s);
 
@@ -248,6 +341,7 @@ void lo_server_free(lo_server s)
 	lo_method it;
 	lo_method next;
 
+	if (s->socket != -1) close(s->socket);
 	freeaddrinfo(s->ai);
 	free(s->hostname);
 	free(s->path);
@@ -267,6 +361,10 @@ void *lo_server_recv_raw(lo_server s, size_t *size)
     int ret;
     void *data = NULL;
 
+#ifdef WIN32
+    if(!initWSock()) return NULL;
+#endif
+
     s->addr_len = sizeof(s->addr);
 
     ret = recvfrom(s->socket, buffer, LO_MAX_MSG_SIZE, 0,
@@ -290,13 +388,23 @@ void *lo_server_recv_raw_stream(lo_server s, size_t *size)
     int32_t read_size;
     int ret;
     void *data = NULL;
-    struct pollfd ps;
     int sock;
 
+#ifdef WIN32
+    if(!initWSock()) return NULL;
+
+    fd_set ps;
+    FD_ZERO(&ps);
+    FD_SET(s->socket,&ps);
+    if(select(1,&ps,NULL,NULL,NULL) == SOCKET_ERROR)
+        return NULL;
+#else
+    struct pollfd ps;
     ps.fd = s->socket;
     ps.events = POLLIN | POLLPRI;
     ps.revents = 0;
     poll(&ps, 1, -1);
+#endif
     sock = accept(s->socket, (struct sockaddr *)&addr, &addr_len);
 
     ret = recv(sock, &read_size, sizeof(read_size), 0);
@@ -322,9 +430,33 @@ void *lo_server_recv_raw_stream(lo_server s, size_t *size)
 
 int lo_server_recv_noblock(lo_server s, int timeout)
 {
-    struct pollfd ps;
     int sched_timeout = lo_server_next_event_delay(s) * 1000;
+#ifdef WIN32
+    fd_set ps;
+    struct timeval stimeout;
+#else
+    struct pollfd ps;
+#endif
 
+#ifdef WIN32
+    int res,to;
+
+    if(!initWSock()) return 0;
+
+    to = timeout > sched_timeout ? sched_timeout : timeout;
+    stimeout.tv_sec = to/1000;
+    stimeout.tv_usec = (to%1000)*1000;
+
+    FD_ZERO(&ps);
+    FD_SET(s->socket,&ps);
+    res = select(1,&ps,NULL,NULL,&stimeout);
+
+    if(res == SOCKET_ERROR)
+        return 0;
+
+    if (res || lo_server_next_event_delay(s) < 0.01)
+	    return lo_server_recv(s);
+#else
     ps.fd = s->socket;
     ps.events = POLLIN | POLLPRI | POLLERR | POLLHUP;
     ps.revents = 0;
@@ -336,6 +468,7 @@ int lo_server_recv_noblock(lo_server s, int timeout)
     if (ps.revents || lo_server_next_event_delay(s) < 0.01) {
 	return lo_server_recv(s);
     }
+#endif
 
     return 0;
 }
@@ -346,16 +479,44 @@ int lo_server_recv(lo_server s)
     size_t size;
     char *path;
     char *types;
+    double sched_time = lo_server_next_event_delay(s);
+#ifdef WIN32
+    fd_set ps;
+    struct timeval stimeout;
+    int res;
+#else
     struct pollfd ps;
-    double sched_time;
-
-    sched_time = lo_server_next_event_delay(s);
+#endif
 
 again:
     if (sched_time > 0.01) {
 	if (sched_time > 10.0) {
 	    sched_time = 10.0;
 	}
+
+#ifdef WIN32
+    if(!initWSock()) return 0;
+
+    ps.fd_count = 1;
+    ps.fd_array[0] = s->socket;
+
+    stimeout.tv_sec = sched_time;
+    stimeout.tv_usec = (sched_time-stimeout.tv_sec)*1.e6;
+	res = select(1,&ps,NULL,NULL,&stimeout);
+	if(res == SOCKET_ERROR) {
+	    return 0;
+	}
+
+	if(!res) {
+	    sched_time = lo_server_next_event_delay(s);
+
+	    if (sched_time > 0.01) {
+		goto again;
+	    }
+
+	    return dispatch_queued(s);
+	}
+#else
 	ps.fd = s->socket;
 	ps.events = POLLIN | POLLPRI | POLLERR | POLLHUP;
 	ps.revents = 0;
@@ -374,10 +535,10 @@ again:
 
 	    return dispatch_queued(s);
 	}
+#endif
     } else {
 	return dispatch_queued(s);
     }
-
     if (s->protocol == LO_TCP) {
 	data = lo_server_recv_raw_stream(s, &size);
     } else {
@@ -393,13 +554,15 @@ again:
     if (!strcmp(path, "#bundle")) {
 	char *pos = types;
 	uint32_t len;
-	lo_pcast64 ats;
 	lo_timetag ts, now;
 
 	lo_timetag_now(&now);
-	ats.nl = lo_otoh64(*((uint64_t *)pos));
-	ts = ats.tt;
-	pos += 8;
+
+	ts.sec = lo_otoh32(*((uint32_t *)pos));
+	pos += 4;
+	ts.frac = lo_otoh32(*((uint32_t *)pos));
+	pos += 4;
+
 	while (pos - (char *)data < size) {
 	    len = lo_otoh32(*((uint32_t *)pos));
 	    pos += 4;
@@ -463,6 +626,7 @@ static void dispatch_method(lo_server s, const char *path, char *types,
     lo_address src = lo_address_new(NULL, NULL);
     char hostname[LO_HOST_SIZE];
     char portname[32];
+    const char *pptr;
 
     free(msg->types);
     msg->types = types;
@@ -498,9 +662,11 @@ static void dispatch_method(lo_server s, const char *path, char *types,
 	    case EAI_NONAME:
 		lo_throw(s, err, "Cannot resolve", path);
 		break;
+#ifndef WIN32
 	    case EAI_SYSTEM:
 		lo_throw(s, err, strerror(err), path);
 		break;
+#endif
 	    default:
 		lo_throw(s, err, "Unknown error", path);
 		break;
@@ -542,7 +708,12 @@ static void dispatch_method(lo_server s, const char *path, char *types,
 		    }
 		}
 
-		ret = it->handler(path, types, argv, argc, msg,
+		/* Send wildcard path to generic handler, expanded path
+		  to others.
+		*/
+		pptr = path;
+		if (it->path) pptr = it->path;
+		ret = it->handler(pptr, types, argv, argc, msg,
 				      it->user_data);
 
 	    } else if (lo_can_coerce_spec(types, it->typespec)) {
@@ -572,7 +743,12 @@ static void dispatch_method(lo_server s, const char *path, char *types,
 		}
 		endian_fixed = 1;
 
-		ret = it->handler(path, it->typespec, argv, argc, msg,
+		/* Send wildcard path to generic handler, expanded path
+		  to others.
+		*/
+		pptr = path;
+		if (it->path) pptr = it->path;
+		ret = it->handler(pptr, it->typespec, argv, argc, msg,
 				      it->user_data);
 		free(argv);
 		free(data_co);
@@ -608,7 +784,11 @@ static void dispatch_method(lo_server s, const char *path, char *types,
 
 		    tmp = malloc(strlen(it->path + len) + 1);
 		    strcpy(tmp, it->path + len);
+#ifdef WIN32
+            sec = strchr(tmp,'/');
+#else
 		    sec = index(tmp, '/');
+#endif
 		    if (sec) *sec = '\0';
 		    slend = sl;
 		    for (slit = sl; slit; slend = slit, slit = slit->next) {
@@ -720,7 +900,7 @@ static int dispatch_queued(lo_server s)
 
 	s->queued = tailhead;
 	head = tailhead;
-    } while (head && lo_timetag_diff(disp_time, head->ts) < FLT_EPSILON);
+    } while (head && lo_timetag_diff(head->ts, disp_time) < FLT_EPSILON);
 
     return 0;
 }
@@ -764,11 +944,55 @@ lo_method lo_server_add_method(lo_server s, const char *path,
     return m;
 }
 
+void lo_server_del_method(lo_server s, const char *path,
+			  const char *typespec)
+{
+    lo_method it, prev, next;
+    int pattern = 0;
+
+    if (!s->first) return;
+    if (path) pattern = strpbrk(path, " #*,?[]{}") != NULL;
+
+    it = s->first;
+    prev = it;
+    while (it) {
+	/* incase we free it */
+	next = it->next;
+
+	/* If paths match or handler is wildcard */
+	if ((it->path == path) ||
+	    (path && it->path && !strcmp(path, it->path)) ||
+	    (pattern && lo_pattern_match(it->path, path))) {
+	    /* If types match or handler is wildcard */
+	    if ((it->typespec == typespec) ||
+		(typespec && it->typespec && !strcmp(typespec, it->typespec))
+	        ) {
+		/* Take care when removing the head. */
+		if (it == s->first) {
+		    s->first = it->next;
+		} else {
+		    prev->next = it->next;
+		}
+		next = it->next;
+		free((void *)it->path);
+		free((void *)it->typespec);
+		free(it);
+		it = prev;
+	    }
+	}
+	prev = it;
+	if (it) it = next;
+    }
+}
+
 int lo_server_get_socket_fd(lo_server s)
 {
     if (s->protocol != LO_UDP &&
-        s->protocol != LO_TCP &&
-        s->protocol != LO_UNIX) {
+        s->protocol != LO_TCP 
+#ifndef WIN32
+        && s->protocol != LO_UNIX
+#endif
+    ) {
         return -1;  /* assume it is not supported */
     }
     return s->socket;
@@ -804,7 +1028,9 @@ char *lo_server_get_url(lo_server s)
 	snprintf(buf, ret+1, "osc.%s://%s:%d/", proto, s->hostname, s->port);
 
 	return buf;
-    } else if (s->protocol == LO_UNIX) {
+    } 
+#ifndef WIN32
+    else if (s->protocol == LO_UNIX) {
 	ret = snprintf(NULL, 0, "osc.unix:///%s", s->path);
 	if (ret <= 0) {
 	    /* this libc is not C99 compliant, guess a size */
@@ -815,7 +1041,7 @@ char *lo_server_get_url(lo_server s)
 
 	return buf;
     }
-
+#endif
     return NULL;
 }
 
