@@ -11,17 +11,23 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  $Id: server_thread.c,v 1.2 2004/11/17 17:34:46 theno23 Exp $
+ *  $Id: server_thread.c,v 1.4 2005/08/09 15:06:05 mhabets Exp $
  */
 
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <netdb.h>
 #include <string.h>
 #include <pthread.h>
 #include <sys/types.h>
+
+#ifdef WIN32
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#else
+#include <netdb.h>
 #include <sys/socket.h>
+#endif
 
 #include "lo_types_internal.h"
 #include "lo/lo.h"
@@ -64,6 +70,12 @@ lo_method lo_server_thread_add_method(lo_server_thread st, const char *path,
     return lo_server_add_method(st->s, path, typespec, h, user_data);
 }
 
+void lo_server_thread_del_method(lo_server_thread st, const char *path,
+			       const char *typespec)
+{
+    lo_server_del_method(st->s, path, typespec);
+}
+
 void lo_server_thread_start(lo_server_thread st)
 {
     if (!st->active) {
@@ -78,7 +90,11 @@ void lo_server_thread_stop(lo_server_thread st)
     if (st->active) {
 	st->active = 0;
 	while (!st->done) {
+#ifdef WIN32
+        Sleep(1);
+#else
 	    usleep(1000);
+#endif
 	}
     }
 }
