@@ -11,7 +11,7 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  $Id: testlo.c,v 1.15 2005/03/02 16:54:43 theno23 Exp $
+ *  $Id: testlo.c,v 1.20 2005/08/19 10:12:35 theno23 Exp $
  */
 
 /*
@@ -30,6 +30,12 @@
 #include "lo/lo.h"
 #include "config.h"
 
+#ifdef WIN32
+#define PATHDELIM "\\"
+#else
+#define PATHDELIM "/"
+#endif
+
 #define TEST(cond) if (!(cond)) { fprintf(stderr, "FAILED " #cond \
 					  " at %s:%d\n", __FILE__, __LINE__); \
 				  exit(1); } \
@@ -54,6 +60,11 @@ static int subtest_reply_count = 0;
 
 char testdata[5] = "ABCDE";
 
+static int jitter_count = 0;
+static float jitter_total = 0.0f;
+static float jitter_max = 0.0f;
+static float jitter_min = 1000.0f;
+
 void exitcheck(void);
 
 void error(int num, const char *m, const char *path);
@@ -119,7 +130,11 @@ int main()
     /* leak check */
     st = lo_server_thread_new(NULL, error);
     lo_server_thread_start(st);
+#ifdef WIN32
+    Sleep(4);
+#else
     usleep(4000);
+#endif
     lo_server_thread_stop(st);
     lo_server_thread_free(st);
     st = lo_server_thread_new(NULL, error);
@@ -191,6 +206,16 @@ int main()
 	printf("passed lo_url_get_protocol() test\n");
     }
     free(protocol);
+    
+    protocol = lo_url_get_protocol("osc.udp://[::ffff:localhost]:9999/a/path/is/here");
+    if (strcmp(protocol, "udp")) {
+	printf("failed lo_url_get_protocol() test1\n");
+	printf("'%s' != 'udp'\n", protocol);
+	exit(1);
+    } else {
+	printf("passed lo_url_get_protocol() test (IPv6)\n");
+    }
+    free(protocol);
 
     host = lo_url_get_hostname("osc.udp://foo.example.com:9999/a/path/is/here");
     if (strcmp(host, "foo.example.com")) {
@@ -221,9 +246,18 @@ int main()
 	printf("passed lo_url_get_port() test\n");
     }
     free(port);
+    
+    port = lo_url_get_port("osc.udp://[::ffff:127.0.0.1]:9999/a/path/is/here");
+    if (strcmp(port, "9999")) {
+	printf("failed lo_url_get_port() test1\n");
+	printf("'%s' != '9999'\n", port);
+	exit(1);
+    } else {
+	printf("passed lo_url_get_port() test (IPv6)\n");
+    }
+    free(port);
     printf("\n");
 
-
     if (lo_blob_datasize(btest) != 5 || lo_blobsize(btest) != 12) {
 	printf("blob is %d (%d) bytes long, should be 5 (12)\n",
                lo_blob_datasize(btest), lo_blobsize(btest));
@@ -246,7 +280,7 @@ int main()
 
     lo_server_thread_add_method(st, "/bundle", NULL,
 				bundle_handler, NULL);
-    lo_server_thread_add_method(st, "/jitter", "t",
+    lo_server_thread_add_method(st, "/jitter", "ti",
 				jitter_handler, NULL);
 
     lo_server_thread_add_method(st, "/pattern/foo", NULL,
@@ -274,12 +308,12 @@ int main()
     lo_server_thread_start(st);
 
     if (lo_send(a, "/foo/bar", "ff", 0.12345678f, 23.0f) == -1) {
-	printf("OSC error %d: %s\n", lo_address_errno(a), lo_address_errstr(a));
+	printf("OSC error A %d: %s\n", lo_address_errno(a), lo_address_errstr(a));
 	exit(1);
     }
 
     if (lo_send(a, "/foo/bar", "ff", 0.12345678f, 23.0f) == -1) {
-	printf("OSC error %d: %s\n", lo_address_errno(a), lo_address_errstr(a));
+	printf("OSC error B %d: %s\n", lo_address_errno(a), lo_address_errstr(a));
 	exit(1);
     }
 
@@ -302,14 +336,18 @@ int main()
     lo_send(a, "/pattern/ba[rz]", "s", "b");
 
     server_url = lo_server_thread_get_url(st);
-    sprintf(cmd, "./subtest %s &", server_url);
+    sprintf(cmd, "." PATHDELIM "subtest %s &", server_url);
     if (system(cmd) != 0) {
 	fprintf(stderr, "Cannot execute subtest command\n");
 	exit(1);
     }
     system(cmd);
 
+#ifdef WIN32
+    Sleep(2000);
+#else
     sleep(2);
+#endif
     TEST(reply_count == 2);
     TEST(pattern_count == 5);
     TEST(subtest_count == 2);
@@ -358,6 +396,7 @@ int main()
     lo_bundle_free(b);
 
     lo_timetag_now(&sched);
+
     sched.sec += 5;
     b = lo_bundle_new(sched);
     m1 = lo_message_new();
@@ -365,43 +404,58 @@ int main()
     lo_message_add_string(m1, "time");
     lo_message_add_string(m1, "test");
     lo_bundle_add_message(b, "/bundle", m1);
+
     lo_send_bundle(a, b);
     lo_message_free(m1);
     lo_bundle_free(b);
 
     lo_send_timestamped(a, sched, "/bundle", "s", "lo_send_timestamped() test");
 
-#if 0
+#define JITTER_ITS 25
     /* jitter tests */
     {
-	lo_timetag stamps[1000];
+	lo_timetag stamps[JITTER_ITS];
 	lo_timetag now;
 	int i;
 
-	for (i=0; i<1000; i++) {
+	for (i=0; i<JITTER_ITS; i++) {
 	    lo_timetag_now(&now);
 	    stamps[i] = now;
 	    stamps[i].sec += 1;
-	    stamps[i].frac += 0x8FFFFFFF;
-	    lo_send_timestamped(a, stamps[i], "/jitter", "t", stamps[i]);
+	    stamps[i].frac = rand();
+	    lo_send_timestamped(a, stamps[i], "/jitter", "ti", stamps[i], i);
 	}
     }
+
+#ifdef WIN32
+    Sleep(2000);
+#else
+    sleep(2);
 #endif
 
     lo_address_free(a);
 
-    sleep(2);
-
     TEST(lo_server_thread_events_pending(st));
 
     while (lo_server_thread_events_pending(st)) {
 	printf("pending events, wait...\n");
+#ifdef WIN32
+	fflush(stdout);
+	Sleep(1000);
+#else
 	sleep(1);
+#endif
     }
     
     TEST(bundle_count == 6);
     printf("\n");
 
+    printf("bundle timing jitter results:\n"
+	   "max jitter = %fs\n"
+	   "avg jitter = %fs\n"
+           "min jitter = %fs\n\n",
+           jitter_max, jitter_total/(float)jitter_count, jitter_min);
+
     server_url = lo_server_get_url(s);
 
     lo_server_add_method(s, NULL, NULL, generic_handler, NULL);
@@ -418,10 +472,15 @@ int main()
 	exit(1);
     }
 
+    /* Delete methods */
+    lo_server_thread_del_method(st, "/coerce", "dfhiSs");
+    lo_server_del_method(s, NULL, NULL);
+
     lo_address_free(a);
     lo_server_free(s);
     free(server_url);
 
+#ifndef WIN32
     { /* UNIX domain tests */
 	lo_address ua;
 	lo_server us;
@@ -438,6 +497,7 @@ int main()
 	lo_server_free(us);
 	lo_address_free(ua);
     }
+#endif
 
     { /* TCP tests */
 	lo_address ta;
@@ -471,7 +531,11 @@ int main()
     lo_address_free(a);
 
     while (!done) {
+#ifdef WIN32
+    Sleep(1);
+#else
 	usleep(1000);
+#endif
     }
 
     lo_server_thread_free(st);
@@ -571,6 +635,7 @@ int lots_handler(const char *path, const char *types, lo_arg **argv, int argc,
 	 argv[6]->t.frac == 0x80000000);
     TEST(types[7] == 'd' && argv[7]->d == 0.9999);
     TEST(types[8] == 'S' && !strcmp(&argv[8]->S, "sym"));
+printf("char: %d\n", argv[9]->c);
     TEST(types[9] == 'c' && argv[9]->c == 'X');
     TEST(types[10] == 'c' && argv[10]->c == 'Y');
     TEST(types[11] == 'T');
@@ -608,12 +673,20 @@ int bundle_handler(const char *path, const char *types, lo_arg **argv, int argc,
 }
 
 int jitter_handler(const char *path, const char *types, lo_arg **argv, int argc,
-		 lo_message data, void *user_data)
+                 lo_message data, void *user_data)
 {
     lo_timetag now;
+    float jitter;
 
     lo_timetag_now(&now);
-    printf("jitter: %f\n", lo_timetag_diff(now, argv[0]->t));
+    jitter = fabs(lo_timetag_diff(now, argv[0]->t));
+    jitter_count++;
+    //printf("jitter: %f\n", jitter);
+    printf("%d expected: %x:%x received %x:%x\n", argv[1]->i, argv[0]->t.sec,
+	   argv[0]->t.frac, now.sec, now.frac);
+    jitter_total += jitter;
+    if (jitter > jitter_max) jitter_max = jitter;
+    if (jitter < jitter_min) jitter_min = jitter;
 
     return 0;
 }
